# frozen_string_literal: true

module Hashtastic
  class MerkleTree
    class LayersCreator
      class << self
        def call(leafs, hashfunc = EthereumSHA3)
          @hashfunc = hashfunc
          @leafs = leafs

          return [['']] if @leafs.empty?

          layers = []
          layers.push(@leafs)

          while layers[layers.length - 1].length > 1
            layers.push(next_layer(layers[layers.length - 1]))
          end

          layers
        end

        private

        def combine_hash(first, second)
          return first if second.nil?
          @hashfunc.call(sort_and_concat(first, second))
        end

        def next_layer(leafs)
          leafs.each_with_object([]).with_index do |(leaf, arr), index|
            arr << combine_hash(leaf, leafs[index + 1]) if index.even?
          end
        end

        def sort_and_concat(first, second)
          [first, second].map { |leaf| Utils.hex_to_ascii(leaf) }.sort.join
        end
      end
    end
  end
end
